@@ -141,8 +141,8 @@
  */
 #ifndef container_of
 #define container_of(ptr, type, member) __extension__ ({	\
-	void *__mptr = (void *)(ptr);				\
-	((type *)(__mptr - offsetof(type, member))); })
+	const __typeof__( ((type *)0)->member ) *__mptr = (ptr); \
+	(type *)( (char *)__mptr - offsetof(type,member) );})
 #endif
 
 #ifndef HAVE_PROGRAM_INVOCATION_SHORT_NAME
