@@ -120,8 +120,8 @@ static int colors_read_schemes(struct ul_color_ctl *cc);
  */
 static int cmp_scheme_name(const void *a0, const void *b0)
 {
-	struct ul_color_scheme	*a = (struct ul_color_scheme *) a0,
-				*b = (struct ul_color_scheme *) b0;
+	const struct ul_color_scheme *a = (const struct ul_color_scheme *) a0,
+				     *b = (const struct ul_color_scheme *) b0;
 	return strcmp(a->name, b->name);
 }
 
@@ -646,7 +646,7 @@ done:
 
 static void termcolors_init_debug(void)
 {
-	__UL_INIT_DEBUG(termcolors, TERMCOLORS_DEBUG_, 0, TERMINAL_COLORS_DEBUG);
+	__UL_INIT_DEBUG_FROM_ENV(termcolors, TERMCOLORS_DEBUG_, 0, TERMINAL_COLORS_DEBUG);
 }
 
 static int colors_terminal_is_ready(void)
