@@ -1,8 +1,13 @@
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 /*
- * Copyright (C) 2008-2009 Karel Zak <kzak@redhat.com>
+ * This file is part of libmount from util-linux project.
  *
- * This file may be redistributed under the terms of the
- * GNU Lesser General Public License.
+ * Copyright (C) 2008-2018 Karel Zak <kzak@redhat.com>
+ *
+ * libmount is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
  */
 
 /**
@@ -840,7 +845,7 @@ int mnt_fs_append_options(struct libmnt_fs *fs, const char *optstr)
 	if (!optstr)
 		return 0;
 
-	rc = mnt_split_optstr((char *) optstr, &u, &v, &f, 0, 0);
+	rc = mnt_split_optstr(optstr, &u, &v, &f, 0, 0);
 	if (rc)
 		return rc;
 
@@ -882,7 +887,7 @@ int mnt_fs_prepend_options(struct libmnt_fs *fs, const char *optstr)
 	if (!optstr)
 		return 0;
 
-	rc = mnt_split_optstr((char *) optstr, &u, &v, &f, 0, 0);
+	rc = mnt_split_optstr(optstr, &u, &v, &f, 0, 0);
 	if (rc)
 		return rc;
 
