@@ -1,10 +1,14 @@
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 /*
- * Copyright (C) 2009 Karel Zak <kzak@redhat.com>
+ * This file is part of libmount from util-linux project.
  *
- * This file may be redistributed under the terms of the
- * GNU Lesser General Public License.
+ * Copyright (C) 2009-2018 Karel Zak <kzak@redhat.com>
+ *
+ * libmount is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
  */
-
 #ifdef HAVE_SCANDIRAT
 #ifndef __USE_GNU
 #define __USE_GNU
@@ -39,32 +43,34 @@ static void parser_cleanup(struct libmnt_parser *pa)
 	memset(pa, 0, sizeof(*pa));
 }
 
-static int next_number(char **s, int *num)
+static const char *next_number(const char *s, int *num, int *ok)
 {
 	char *end = NULL;
 
 	assert(num);
 	assert(s);
+	assert(ok);
 
-	*s = (char *) skip_blank(*s);
-	if (!**s)
-		return -1;
-	*num = strtol(*s, &end, 10);
-	if (end == NULL || *s == end)
-	       return -1;
+	*ok = 0;
+	s = skip_blank(s);
+	if (!s || !*s)
+		return s;
 
-	*s = end;
+	*num = strtol(s, &end, 10);
+	if (end == NULL || s == end)
+	       return s;
 
 	/* valid end of number is a space or a terminator */
 	if (*end == ' ' || *end == '\t' || *end == '\0')
-		return 0;
-	return -1;
+		*ok = 1;
+
+	return end;
 }
 
 /*
  * Parses one line from {fs,m}tab
  */
-static int mnt_parse_table_line(struct libmnt_fs *fs, char *s)
+static int mnt_parse_table_line(struct libmnt_fs *fs, const char *s)
 {
 	int rc, n = 0, xrc;
 	char *src = NULL, *fstype = NULL, *optstr = NULL;
@@ -119,16 +125,22 @@ static int mnt_parse_table_line(struct libmnt_fs *fs, char *s)
 	fs->passno = fs->freq = 0;
 
 	if (xrc == 4 && n)
-		s = (char *) skip_blank(s + n);
+		s = skip_blank(s + n);
 	if (xrc == 4 && *s) {
-		if (next_number(&s, &fs->freq) != 0) {
-			if (*s) {
+		int ok = 0;
+
+		s = next_number(s, &fs->freq, &ok);
+		if (!ok) {
+			if (s && *s) {
 				DBG(TAB, ul_debug("tab parse error: [freq]"));
 				rc = -EINVAL;
 			}
-		} else if (next_number(&s, &fs->passno) != 0 && *s) {
-			DBG(TAB, ul_debug("tab parse error: [passno]"));
-			rc = -EINVAL;
+		} else {
+			s = next_number(s, &fs->passno, &ok);
+			if (!ok && s && *s) {
+				DBG(TAB, ul_debug("tab parse error: [passno]"));
+				rc = -EINVAL;
+			}
 		}
 	}
 
@@ -138,11 +150,12 @@ static int mnt_parse_table_line(struct libmnt_fs *fs, char *s)
 /*
  * Parses one line from a mountinfo file
  */
-static int mnt_parse_mountinfo_line(struct libmnt_fs *fs, char *s)
+static int mnt_parse_mountinfo_line(struct libmnt_fs *fs, const char *s)
 {
 	int rc, end = 0;
 	unsigned int maj, min;
-	char *fstype = NULL, *src = NULL, *p;
+	char *fstype = NULL, *src = NULL;
+	const char *p;
 
 	rc = sscanf(s,	"%d "		/* (1) id */
 			"%d "		/* (2) parent */
@@ -173,13 +186,36 @@ static int mnt_parse_mountinfo_line(struct libmnt_fs *fs, char *s)
 		fs->opt_fields = strndup(s + 1, p - s - 1);
 	s = p + 3;
 
-	rc += sscanf(s,	UL_SCNsA" "	/* (8) FS type */
-			UL_SCNsA" "	/* (9) source */
-			UL_SCNsA,	/* (10) fs options (fs specific) */
+	end = 0;
+	rc += sscanf(s,	UL_SCNsA"%n",	/* (8) FS type */
 
 			&fstype,
-			&src,
-			&fs->fs_optstr);
+			&end);
+
+	if (rc >= 8 && end > 0)
+		s += end;
+	if (s[0] == ' ')
+		s++;
+
+	/* (9) source can unfortunately be an empty string "" and scanf does
+	 * not work well with empty string. Test with:
+	 *     $ sudo mount -t tmpfs "" /tmp/bb
+	 *     $ mountpoint /tmp/bb
+	 * */
+	if (s[0] == ' ') {
+		src = strdup("");
+		s++;
+		rc++;
+		rc += sscanf(s,	UL_SCNsA,	/* (10) fs options (fs specific) */
+
+				&fs->fs_optstr);
+	} else {
+		rc += sscanf(s,	UL_SCNsA" "	/* (9) source */
+				UL_SCNsA,	/* (10) fs options (fs specific) */
+
+				&src,
+				&fs->fs_optstr);
+	}
 
 	if (rc >= 10) {
 		size_t sz;
@@ -240,7 +276,7 @@ static int mnt_parse_utab_line(struct libmnt_fs *fs, const char *s)
 	assert(!fs->target);
 
 	while (p && *p) {
-		char *end = NULL;
+		const char *end = NULL;
 
 		while (*p == ' ') p++;
 		if (!*p)
@@ -294,7 +330,7 @@ enomem:
 /*
  * Parses one line from /proc/swaps
  */
-static int mnt_parse_swaps_line(struct libmnt_fs *fs, char *s)
+static int mnt_parse_swaps_line(struct libmnt_fs *fs, const char *s)
 {
 	uintmax_t fsz, usz;
 	int rc;
@@ -351,7 +387,7 @@ static int mnt_parse_swaps_line(struct libmnt_fs *fs, char *s)
  *
  * mountinfo: "<number> <number> ... "
  */
-static int guess_table_format(char *line)
+static int guess_table_format(const char *line)
 {
 	unsigned int a, b;
 
@@ -366,9 +402,9 @@ static int guess_table_format(char *line)
 	return MNT_FMT_FSTAB;		/* fstab, mtab or /proc/mounts */
 }
 
-static int is_comment_line(char *line)
+static int is_comment_line(const char *line)
 {
-	char *p	= (char *) skip_blank(line);
+	const char *p = skip_blank(line);
 
 	if (p && (*p == '#' || *p == '\n'))
 		return 1;
