@@ -36,6 +36,9 @@ UL_DEBUG_DECLARE_MASK(libsmartcols);
 #define ON_DBG(m, x)	__UL_DBG_CALL(libsmartcols, SCOLS_DEBUG_, m, x)
 #define DBG_FLUSH	__UL_DBG_FLUSH(libsmartcols, SCOLS_DEBUG_)
 
+#define UL_DEBUG_CURRENT_MASK	UL_DEBUG_MASK(libsmartcols)
+#include "debugobj.h"
+
 /*
  * Generic iterator
  */
@@ -83,6 +86,8 @@ struct libscols_column {
 	size_t	width_treeart;	/* size of the tree ascii art */
 	double	width_hint;	/* hint (N < 1 is in percent of termwidth) */
 
+	int	json_type;	/* SCOLS_JSON_* */
+
 	int	flags;
 	int	is_extreme;
 	char	*color;		/* default column color */
