@@ -61,6 +61,7 @@ static void check_padding_debug(struct libscols_table *tb)
 struct libscols_table *scols_new_table(void)
 {
 	struct libscols_table *tb;
+	int c, l;
 
 	tb = calloc(1, sizeof(struct libscols_table));
 	if (!tb)
@@ -68,7 +69,10 @@ struct libscols_table *scols_new_table(void)
 
 	tb->refcount = 1;
 	tb->out = stdout;
-	tb->termwidth = get_terminal_width(80);
+
+	get_terminal_dimension(&c, &l);
+	tb->termwidth  = c > 0 ? c : 80;
+	tb->termheight = l > 0 ? l : 24;
 
 	INIT_LIST_HEAD(&tb->tb_lines);
 	INIT_LIST_HEAD(&tb->tb_columns);
@@ -150,7 +154,7 @@ const char *scols_table_get_name(const struct libscols_table *tb)
  * title output alignment on non-tty is hardcoded to 80 output chars. For the
  * regular terminal it's based on terminal width.
  *
- * Returns: Title of the table, or %NULL in case of blank title.
+ * Returns: Title of the table, or NULL in case of blank title.
  *
  * Since: 2.28
  */
@@ -181,7 +185,7 @@ int scols_table_add_column(struct libscols_table *tb, struct libscols_column *cl
 	if (cl->flags & SCOLS_FL_TREE)
 		tb->ntreecols++;
 
-	DBG(TAB, ul_debugobj(tb, "add column %p", cl));
+	DBG(TAB, ul_debugobj(tb, "add column"));
 	list_add_tail(&cl->cl_columns, &tb->tb_columns);
 	cl->seqnum = tb->ncols++;
 	cl->table = tb;
@@ -221,7 +225,7 @@ int scols_table_remove_column(struct libscols_table *tb,
 	if (cl->flags & SCOLS_FL_TREE)
 		tb->ntreecols--;
 
-	DBG(TAB, ul_debugobj(tb, "remove column %p", cl));
+	DBG(TAB, ul_debugobj(tb, "remove column"));
 	list_del_init(&cl->cl_columns);
 	tb->ncols--;
 	cl->table = NULL;
@@ -255,7 +259,7 @@ int scols_table_remove_columns(struct libscols_table *tb)
  * scols_table_move_column:
  * @tb: table
  * @pre: column before the column
- * @cl: colum to move
+ * @cl: column to move
  *
  * Move the @cl behind @pre. If the @pre is NULL then the @col is the first
  * column in the table.
@@ -490,6 +494,10 @@ FILE *scols_table_get_stream(const struct libscols_table *tb)
  * The @reduce must be smaller than terminal width, otherwise it's silently
  * ignored. The reduction is not applied when STDOUT_FILENO is not terminal.
  *
+ * Note that after output initialization (scols_table_print_* calls) the width
+ * will be reduced, this behavior affects subsequenced scols_table_get_termwidth()
+ * calls.
+ *
  * Returns: 0, a negative value in case of an error.
  */
 int scols_table_reduce_termwidth(struct libscols_table *tb, size_t reduce)
@@ -540,7 +548,7 @@ struct libscols_column *scols_table_get_column(struct libscols_table *tb,
  */
 int scols_table_add_line(struct libscols_table *tb, struct libscols_line *ln)
 {
-	if (!tb || !ln || tb->ncols == 0)
+	if (!tb || !ln)
 		return -EINVAL;
 
 	if (tb->ncols > ln->ncells) {
@@ -549,7 +557,7 @@ int scols_table_add_line(struct libscols_table *tb, struct libscols_line *ln)
 			return rc;
 	}
 
-	DBG(TAB, ul_debugobj(tb, "add line %p", ln));
+	DBG(TAB, ul_debugobj(tb, "add line"));
 	list_add_tail(&ln->ln_lines, &tb->tb_lines);
 	ln->seqnum = tb->nlines++;
 	scols_ref_line(ln);
@@ -572,7 +580,7 @@ int scols_table_remove_line(struct libscols_table *tb,
 	if (!tb || !ln)
 		return -EINVAL;
 
-	DBG(TAB, ul_debugobj(tb, "remove line %p", ln));
+	DBG(TAB, ul_debugobj(tb, "remove line"));
 	list_del_init(&ln->ln_lines);
 	tb->nlines--;
 	scols_unref_line(ln);
@@ -649,7 +657,7 @@ struct libscols_line *scols_table_new_line(struct libscols_table *tb,
 {
 	struct libscols_line *ln;
 
-	if (!tb || !tb->ncols)
+	if (!tb)
 		return NULL;
 
 	ln = scols_new_line();
@@ -716,7 +724,7 @@ struct libscols_table *scols_copy_table(struct libscols_table *tb)
 	if (!ret)
 		return NULL;
 
-	DBG(TAB, ul_debugobj(tb, "copy into %p", ret));
+	DBG(TAB, ul_debugobj(tb, "copy"));
 
 	if (tb->symbols)
 		scols_table_set_symbols(ret, tb->symbols);
@@ -830,14 +838,14 @@ int scols_table_set_symbols(struct libscols_table *tb,
 
 	/* remove old */
 	if (tb->symbols) {
-		DBG(TAB, ul_debugobj(tb, "remove symbols %p reference", tb->symbols));
+		DBG(TAB, ul_debugobj(tb, "remove symbols reference"));
 		scols_unref_symbols(tb->symbols);
 		tb->symbols = NULL;
 	}
 
 	/* set new */
 	if (sy) {					/* ref user defined */
-		DBG(TAB, ul_debugobj(tb, "set symbols so %p", sy));
+		DBG(TAB, ul_debugobj(tb, "set symbols"));
 		tb->symbols = sy;
 		scols_ref_symbols(sy);
 	}
@@ -866,6 +874,10 @@ struct libscols_symbols *scols_table_get_symbols(const struct libscols_table *tb
  * re-printing the same line more than once (e.g. progress bar). Don't use it
  * if you're not sure.
  *
+ * Note that for the last line in the table the separator is disabled at all.
+ * The library differentiate between table terminator and line terminator
+ * (although for standard output \n byte is used in both cases).
+ *
  * Returns: 0 on success, negative number in case of an error.
  */
 int scols_table_enable_nolinesep(struct libscols_table *tb, int enable)
@@ -1024,6 +1036,28 @@ int scols_table_enable_noheadings(struct libscols_table *tb, int enable)
 	return 0;
 }
 
+/**
+ * scols_table_enable_header_repeat:
+ * @tb: table
+ * @enable: 1 or 0
+ *
+ * Enable/disable header line repeat. The header line is printed only once by
+ * default.  Note that the flag will be silently ignored and disabled if the
+ * output is not on terminal or output format is JSON, raw, etc.
+ *
+ * Returns: 0 on success, negative number in case of an error.
+ *
+ * Since: 2.31
+ */
+int scols_table_enable_header_repeat(struct libscols_table *tb, int enable)
+{
+	if (!tb)
+		return -EINVAL;
+	DBG(TAB, ul_debugobj(tb, "header-repeat: %s", enable ? "ENABLE" : "DISABLE"));
+	tb->header_repeat = enable ? 1 : 0;
+	return 0;
+}
+
 /**
  * scols_table_enable_maxout:
  * @tb: table
@@ -1076,6 +1110,39 @@ int scols_table_is_nowrap(const struct libscols_table *tb)
 	return tb->no_wrap;
 }
 
+/**
+ * scols_table_enable_noencoding:
+ * @tb: table
+ * @enable: 1 or 0
+ *
+ * The library encode non-printable and control chars by \xHEX by default.
+ *
+ * Returns: 0 on success, negative number in case of an error.
+ *
+ * Since: 2.31
+ */
+int scols_table_enable_noencoding(struct libscols_table *tb, int enable)
+{
+	if (!tb)
+		return -EINVAL;
+	DBG(TAB, ul_debugobj(tb, "encoding: %s", enable ? "ENABLE" : "DISABLE"));
+	tb->no_encode = enable ? 1 : 0;
+	return 0;
+}
+
+/**
+ * scols_table_is_noencoding:
+ * @tb: a pointer to a struct libscols_table instance
+ *
+ * Returns: 1 if encoding is disabled.
+ *
+ * Since: 2.31
+ */
+int scols_table_is_noencoding(const struct libscols_table *tb)
+{
+	return tb->no_encode;
+}
+
 /**
  * scols_table_colors_wanted:
  * @tb: table
@@ -1120,6 +1187,19 @@ int scols_table_is_noheadings(const struct libscols_table *tb)
 	return tb->no_headings;
 }
 
+/**
+ * scols_table_is_header_repeat
+ * @tb: table
+ *
+ * Returns: 1 if header repeat is enabled.
+ *
+ * Since: 2.31
+ */
+int scols_table_is_header_repeat(const struct libscols_table *tb)
+{
+	return tb->header_repeat;
+}
+
 /**
  * scols_table_is_export:
  * @tb: table
@@ -1422,9 +1502,41 @@ int scols_table_set_termwidth(struct libscols_table *tb, size_t width)
  * scols_table_get_termwidth
  * @tb: table
  *
- * Returns: terminal width or a negative value in case of an error.
+ * Returns: terminal width.
  */
 size_t scols_table_get_termwidth(const struct libscols_table *tb)
 {
 	return tb->termwidth;
 }
+
+/**
+ * scols_table_set_termheight
+ * @tb: table
+ * @height: terminal height (number of lines)
+ *
+ * The library automatically detects terminal height or defaults to 24 lines if
+ * detections is unsuccessful. This function override this behaviour.
+ *
+ * Returns: 0, a negative value in case of an error.
+ *
+ * Since: 2.31
+ */
+int scols_table_set_termheight(struct libscols_table *tb, size_t height)
+{
+	DBG(TAB, ul_debugobj(tb, "set terminatl height: %zu", height));
+	tb->termheight = height;
+	return 0;
+}
+
+/**
+ * scols_table_get_termheight
+ * @tb: table
+ *
+ * Returns: terminal height (number of lines).
+ *
+ * Since: 2.31
+ */
+size_t scols_table_get_termheight(const struct libscols_table *tb)
+{
+	return tb->termheight;
+}
