@@ -462,12 +462,12 @@ static unsigned long long swap_get_size(const struct swap_device *dev,
 {
 	unsigned int last_page = 0;
 	const unsigned int swap_version = SWAP_VERSION;
-	struct swap_header_v1_2 *s;
+	const struct swap_header_v1_2 *s;
 
 	assert(dev);
 	assert(dev->pagesize > 0);
 
-	s = (struct swap_header_v1_2 *) hdr;
+	s = (const struct swap_header_v1_2 *) hdr;
 
 	if (s->version == swap_version)
 		last_page = s->last_page;
@@ -479,7 +479,7 @@ static unsigned long long swap_get_size(const struct swap_device *dev,
 
 static void swap_get_info(struct swap_device *dev, const char *hdr)
 {
-	struct swap_header_v1_2 *s = (struct swap_header_v1_2 *) hdr;
+	const struct swap_header_v1_2 *s = (const struct swap_header_v1_2 *) hdr;
 
 	assert(dev);
 
@@ -846,7 +846,8 @@ int main(int argc, char *argv[])
 		BYTES_OPTION = CHAR_MAX + 1,
 		NOHEADINGS_OPTION,
 		RAW_OPTION,
-		SHOW_OPTION
+		SHOW_OPTION,
+		OPT_LIST_TYPES
 	};
 
 	static const struct option long_opts[] = {
@@ -861,6 +862,7 @@ int main(int argc, char *argv[])
 		{ "verbose",    no_argument,       NULL, 'v'               },
 		{ "version",    no_argument,       NULL, 'V'               },
 		{ "show",       optional_argument, NULL, SHOW_OPTION       },
+		{ "output-all", no_argument,       NULL, OPT_LIST_TYPES    },
 		{ "noheadings", no_argument,       NULL, NOHEADINGS_OPTION },
 		{ "raw",        no_argument,       NULL, RAW_OPTION        },
 		{ "bytes",      no_argument,       NULL, BYTES_OPTION      },
@@ -951,6 +953,10 @@ int main(int argc, char *argv[])
 			}
 			ctl.show = 1;
 			break;
+		case OPT_LIST_TYPES:
+			for (ctl.ncolumns = 0; (size_t)ctl.ncolumns < ARRAY_SIZE(infos); ctl.ncolumns++)
+				ctl.columns[ctl.ncolumns] = ctl.ncolumns;
+			break;
 		case NOHEADINGS_OPTION:
 			ctl.no_heading = 1;
 			break;
